/*
 * The MIT License
 *
 * Copyright (c) 2015 Juan Cruz Viotti. https://github.com/jviotti
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

/**
 * @module removedrive
 */

var child_process = require('child_process');
var path = require('path');
var _ = require('lodash');

var REMOVE_DRIVE_EXE_PATH = path.join(__dirname, '..', 'bin', 'RemoveDrive.exe');

/**
 * @summary Eject a drive by it's letter
 * @function
 * @public
 *
 * @description
 * Running this function requires administration permissions.
 *
 * @param {String} driveLetter - drive letter
 * @param {Function} callback - callback (error)
 *
 * @example
 * removedrive.eject('D', function(error) {
 *   if (error) throw error;
 * });
 */
exports.eject = function(driveLetter, callback) {

  if (_.last(driveLetter) !== ':') {
    driveLetter += ':';
  }

  child_process.execFile(REMOVE_DRIVE_EXE_PATH, [
    driveLetter
  ], function(error, stdout, stderr) {
    if (error) {
      return callback(error);
    }

    if (!_.isEmpty(stderr.trim())) {
      return callback(new Error(stderr.trim()));
    }

    return callback();
  });
};
